@@ -6,6 +6,7 @@
  *
  */
 
+#define _WIN32_WINNT 0x0502
 #define _CRT_NON_CONFORMING_SWPRINTFS 
 #define _CRT_SECURE_NO_DEPRECATE
 
@@ -61,7 +62,7 @@ tstring SERVER::getRegistryKeyString( const tstring& subKey, const tstring& valu
    tstring result;
    for(;;)
    {
-      if( RegOpenKeyEx( baseKey, keyName.c_str(), 0, KEY_READ, &key ) == ERROR_SUCCESS ) 
+      if( RegOpenKeyEx( baseKey, keyName.c_str(), 0, KEY_READ | KEY_WOW64_64KEY, &key ) == ERROR_SUCCESS )
       {
          DWORD neededSizeInBytes = 0;
          if (RegQueryValueEx(key, value.c_str(), 0, 0, 0, &neededSizeInBytes) == ERROR_SUCCESS) 
@@ -243,16 +244,20 @@ SERVER::recent_files()
    LOG();
    if ( m_pRecentFiles==0 )
    {
-      MESSAGELOG(TEXT("Reading history from registry..."));
       m_pRecentFiles = new std::list<tstring>;
-      for( int i=0; i<32; ++i )  // Max history size
-      {
-         TCHAR numAsString[10];
-         _sntprintf( numAsString, 10, TEXT("%d"), i );
-         tstring historyItem = getRegistryKeyString( TEXT("history"), numAsString );
-         if ( ! historyItem.empty() )
-            m_pRecentFiles->push_back( historyItem );
-      }
+   }
+   else
+   {
+      m_pRecentFiles->clear();
+   }
+   MESSAGELOG(TEXT("Reading history from registry..."));
+   for( int i=0; i<32; ++i )  // Max history size
+   {
+      TCHAR numAsString[10];
+      _sntprintf( numAsString, 10, TEXT("%d"), i );
+      tstring historyItem = getRegistryKeyString( TEXT("history"), numAsString );
+      if ( ! historyItem.empty() )
+         m_pRecentFiles->push_back( historyItem );
    }
    return *m_pRecentFiles;
 }
@@ -260,11 +265,11 @@ SERVER::recent_files()
 void
 SERVER::save_history() const 
 {
-   if( m_pRecentFiles && !m_pRecentFiles->empty() ) 
+   if( m_pRecentFiles ) 
    {
       HKEY key;
       if( RegCreateKeyEx(HKEY_CURRENT_USER, (m_registryBaseName + TEXT("\\history")).c_str(), 0, 0, 
-                         REG_OPTION_NON_VOLATILE, KEY_WRITE, 0, &key, 0) == ERROR_SUCCESS ) 
+                         REG_OPTION_NON_VOLATILE, KEY_WRITE | KEY_WOW64_64KEY, 0, &key, 0) == ERROR_SUCCESS )
       {
          LOG();
          //DWORD len = MAX_PATH;
@@ -283,10 +288,16 @@ SERVER::save_history() const
                FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, 0,
                   GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                   (LPTSTR) &message, 0, 0);
-               MessageBox(0, message, TEXT("Save history failed"), MB_OK | MB_ICONINFORMATION);
+               MessageBox(0, message, TEXT("KDiff3-diff-ext: Save history failed"), MB_OK | MB_ICONINFORMATION);
                LocalFree(message);
             }
          }
+         for(; n<32; ++n )
+         {
+            TCHAR numAsString[10];
+            _sntprintf( numAsString, 10, TEXT("%d"), n );
+            RegDeleteValue(key, numAsString ); 
+         }
 
          RegCloseKey(key);
       }
