@@ -11,21 +11,29 @@
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
 
-#include <QDialog>
+#include "progress.h"
+
 #include <QDateTime>
-#include <QEventLoop>
-#include <QLabel>
-#include <QProgressBar>
 
 #include <kprogressdialog.h>
 #include <kio/job.h>
 #include <kio/jobclasses.h>
+#include <kurl.h>
 
-#include <list>
 
 bool wildcardMultiMatch( const QString& wildcard, const QString& testString, bool bCaseSensitive );
 
 class t_DirectoryList;
+class QFileInfo;
+
+class ProgressProxyExtender: public ProgressProxy
+{
+  Q_OBJECT
+
+public slots:
+  void slotListDirInfoMessage( KJob*, const QString& msg );
+  void slotPercent( KJob*, unsigned long percent );
+};
 
 class FileAccess
 {
@@ -33,6 +41,8 @@ public:
    FileAccess();
    ~FileAccess();
    FileAccess( const QString& name, bool bWantToWrite=false ); // name: local file or dirname or url (when supported)
+   FileAccess(const FileAccess& other);
+   const FileAccess& operator=(const FileAccess& other);
    void setFile( const QString& name, bool bWantToWrite=false );
 
    bool isValid() const;
@@ -48,9 +58,9 @@ public:
    bool isHidden() const;
    QString readLink() const;
 
-   QDateTime   created()       const;
+   //QDateTime   created()       const;
    QDateTime   lastModified()  const;
-   QDateTime   lastRead()      const;
+   //QDateTime   lastRead()      const;
 
    QString fileName() const; // Just the name-part of the path, without parent directories
    QString filePath() const; // The path-string that was used during construction
@@ -83,34 +93,34 @@ public:
 
    void addPath( const QString& txt );
    QString getStatusText();
+
+   FileAccess* parent() const; // !=0 for listDir-results, but only valid if the parent was not yet destroyed.
+   void setSharedName( const QString& name ); // might reduce memory consumption
 private:
    void setUdsEntry( const KIO::UDSEntry& e );
-   KUrl m_url;
-   bool m_bLocal;
-   bool m_bValidData;
+   void setFile( const QFileInfo& fi, FileAccess* pParent );
+   void setStatusText( const QString& s );
 
+   class Data;
+   Data* d();
+   const Data* d() const;
+   void createData();
+
+   union
+   {
+      Data* m_pData;
+      FileAccess* m_pParent;
+   };
+   QString m_filePath; // might be absolute or relative if m_pParent!=0
    qint64 m_size;
    QDateTime m_modificationTime;
-   QDateTime m_accessTime;
-   QDateTime m_creationTime;
-   bool m_bReadable;
-   bool m_bWritable;
-   bool m_bExecutable;
-   bool m_bExists;
-   bool m_bFile;
-   bool m_bDir;
-   bool m_bSymLink;
-   bool m_bHidden;
-   long m_fileType; // for testing only
-
-   QString m_linkTarget;
-   QString m_user;
-   QString m_group;
-   QString m_name;
-   QString m_path;
-   QString m_absoluteFilePath;
-   QString m_localCopy;
-   QString m_statusText;  // Might contain an error string, when the last operation didn't succeed.
+   bool m_bSymLink  : 1;
+   bool m_bFile     : 1;
+   bool m_bDir      : 1;
+   bool m_bExists   : 1;
+   bool m_bWritable : 1;
+   bool m_bHidden   : 1;
+   bool m_bUseData  : 1;
 
    friend class FileAccessJobHandler;
 };
@@ -165,104 +175,9 @@ private slots:
    void slotGetData(KJob*,const QByteArray&);
    void slotPutData(KIO::Job*, QByteArray&);
 
-   void slotListDirInfoMessage( KJob*, const QString& msg );
    void slotListDirProcessNewEntries( KIO::Job *, const KIO::UDSEntryList& l );
-
-   void slotPercent( KJob* pJob, unsigned long percent );
 };
 
-class ProgressDialog : public QDialog
-{
-   Q_OBJECT
-public:
-   ProgressDialog( QWidget* pParent );
-
-   void setStayHidden( bool bStayHidden );
-   void setInformation( const QString& info, bool bRedrawUpdate=true );
-   void setInformation( const QString& info, double dCurrent, bool bRedrawUpdate=true );
-   void setCurrent( double dCurrent, bool bRedrawUpdate=true  );
-   void step( bool bRedrawUpdate=true );
-   void setMaxNofSteps( int dMaxNofSteps );
-   void push();
-   void pop(bool bRedrawUpdate=true);
-
-   // The progressbar goes from 0 to 1 usually.
-   // By supplying a subrange transformation the subCurrent-values
-   // 0 to 1 will be transformed to dMin to dMax instead.
-   // Requirement: 0 < dMin < dMax < 1
-   void setRangeTransformation( double dMin, double dMax );
-   void setSubRangeTransformation( double dMin, double dMax );
-
-   void exitEventLoop();
-   void enterEventLoop( KJob* pJob, const QString& jobInfo );
-
-   bool wasCancelled();
-   void show();
-   void hide();
-   
-   virtual void timerEvent(QTimerEvent*);
-private:
-
-   struct ProgressLevelData
-   {
-      ProgressLevelData()
-      {
-         m_dCurrent=0; m_maxNofSteps=1; m_dRangeMin=0; m_dRangeMax=1; 
-         m_dSubRangeMin = 0; m_dSubRangeMax = 1;
-      }
-      double m_dCurrent;
-      int    m_maxNofSteps;     // when step() is used.
-      double m_dRangeMax;
-      double m_dRangeMin;
-      double m_dSubRangeMax;
-      double m_dSubRangeMin;
-   };
-   std::list<ProgressLevelData> m_progressStack;
-   
-   int m_progressDelayTimer;
-   std::list<QEventLoop*> m_eventLoopStack;
-
-   QProgressBar* m_pProgressBar;
-   QProgressBar* m_pSubProgressBar;
-   QLabel* m_pInformation;
-   QLabel* m_pSubInformation;
-   QLabel* m_pSlowJobInfo;
-   QPushButton* m_pAbortButton;
-   void recalc(bool bRedrawUpdate);
-   QTime m_t1;
-   QTime m_t2;
-   bool m_bWasCancelled;
-   KJob* m_pJob;
-   QString m_currentJobInfo;  // Needed if the job doesn't stop after a reasonable time.
-   bool m_bStayHidden;
-protected:
-   virtual void reject();
-private slots:
-   void delayedHide();
-   void slotAbort();
-};
-
-// When using the ProgressProxy you need not take care of the push and pop, except when explicit.
-class ProgressProxy
-{
-public:
-   ProgressProxy();
-   ~ProgressProxy();
-   
-   void setInformation( const QString& info, bool bRedrawUpdate=true );
-   void setInformation( const QString& info, double dCurrent, bool bRedrawUpdate=true );
-   void setCurrent( double dCurrent, bool bRedrawUpdate=true  );
-   void step( bool bRedrawUpdate=true );
-   void setMaxNofSteps( int dMaxNofSteps );
-   bool wasCancelled();
-   void setRangeTransformation( double dMin, double dMax );
-   void setSubRangeTransformation( double dMin, double dMax );
-private:
-};
-
-extern ProgressDialog* g_pProgressDialog;
-
-
 
 #endif
 
