@@ -300,18 +300,16 @@ class KDiff3App : public QSplitter
    bool m_bTimerBlock;      // Synchronisation
 
    OptionDialog* m_pOptionDialog;
+   Options*      m_pOptions;
    FindDialog*   m_pFindDialog;
 
-   void init( bool bAuto=false, TotalDiffStatus* pTotalDiffStatus=0, bool bLoadFiles=true );
-
+   void init( bool bAuto=false, TotalDiffStatus* pTotalDiffStatus=0, bool bLoadFiles=true, bool bUseCurrentEncoding = false);
 
    virtual bool eventFilter( QObject* o, QEvent* e );
    virtual void resizeEvent(QResizeEvent*);
 
    bool improveFilenames(bool bCreateNewInstance);
 
-   bool runDiff( const LineData* p1, int size1, const LineData* p2, int size2, DiffList& diffList, int winIdx1, int winIdx2 );
-   bool runDiff( const LineData* p1, int size1, const LineData* p2, int size2, DiffList& diffList );
    bool canContinue();
 
    void choose(int choice);
@@ -326,6 +324,8 @@ class KDiff3App : public QSplitter
    void recalcWordWrap(int nofVisibleColumns=-1);
    void setHScrollBarRange();
 
+   int m_iCumulativeWheelDelta;
+
 public slots:
    void resizeDiffTextWindow(int newWidth, int newHeight);
    void resizeMergeResultWindow();
@@ -399,8 +399,10 @@ public slots:
    void slotJoinDiffs();
    void slotAddManualDiffHelp();
    void slotClearManualDiffHelpList();
-
    void slotNoRelevantChangesDetected();
+   void slotEncodingChangedA(QTextCodec*);
+   void slotEncodingChangedB(QTextCodec*);
+   void slotEncodingChangedC(QTextCodec*);
 };
 
 #endif // KDIFF3_H
