@@ -27,6 +27,7 @@
 #include <QTranslator>
 #include <QLocale>
 #include <QFont>
+#include <QClipboard>
 #include <vector>
 
 #ifdef KREPLACEMENTS_H
@@ -86,7 +87,7 @@ static bool isOptionUsed(const QString& s, int argc, char* argv[])
 {
    for(int j=0; j<argc; ++j )
    {
-      if( "-"+s == argv[j] || "--"+s==argv[j] )
+      if( QString("-"+s) == argv[j] || QString("--"+s)==argv[j] )
       {
          return true;
       }
@@ -163,8 +164,10 @@ int main(int argc, char *argv[])
    QByteArray appVersion = QByteArray( VERSION );
    if ( sizeof(void*)==8 )
       appVersion += " (64 bit)";
+   else if ( sizeof(void*)==4 )
+       appVersion += " (32 bit)";
    const KLocalizedString description = ki18n("Tool for Comparison and Merge of Files and Directories");
-   const KLocalizedString copyright = ki18n("(c) 2002-2011 Joachim Eibl");
+   const KLocalizedString copyright = ki18n("(c) 2002-2012 Joachim Eibl");
    const QByteArray& homePage = "http://kdiff3.sourceforge.net/";
    const QByteArray& bugsAddress = "joachim.eibl" "@" "gmx.de";
    KAboutData aboutData( appName, appCatalog, i18nName, 
@@ -188,6 +191,8 @@ int main(int argc, char *argv[])
    aboutData.addCredit(ki18n("Valentin Rusu"), ki18n("KDE4 porting"), QByteArray("kde" "@" "rusu.info"));
    aboutData.addCredit(ki18n("Albert Astals Cid"), ki18n("KDE4 porting"), QByteArray("aacid" "@" "kde.org"));
    aboutData.addCredit(ki18n("Silvan Scherrer"), ki18n("OS2 port") );
+   aboutData.addCredit(ki18n("Nigel Stewart"), ki18n("Cygwin support") );
+   aboutData.addCredit(ki18n("Maurice van der Pot"), ki18n("Framework for automated tests") );
 
    aboutData.addCredit(ki18n("+ Many thanks to those who reported bugs and contributed ideas!"));
 
@@ -200,7 +205,7 @@ int main(int argc, char *argv[])
    options.add( "o" ).add( "output file", ki18n("Output file. Implies -m. E.g.: -o newfile.txt"));
    options.add( "out file",    ki18n("Output file, again. (For compatibility with certain tools.)") );
    options.add( "auto",        ki18n("No GUI if all conflicts are auto-solvable. (Needs -o file)") );
-   options.add( "qall",        ki18n("Don't solve conflicts automatically. (For compatibility...)") );
+   options.add( "qall",        ki18n("Don't solve conflicts automatically.") );
    options.add( "L1 alias1",   ki18n("Visible name replacement for input file 1 (base).") );
    options.add( "L2 alias2",   ki18n("Visible name replacement for input file 2.") );
    options.add( "L3 alias3",   ki18n("Visible name replacement for input file 3.") );
@@ -242,17 +247,22 @@ int main(int argc, char *argv[])
    }
 #endif
 
+#ifndef QT_NO_SESSIONMANAGER
   if (app.isSessionRestored())
   {
      RESTORE(KDiff3Shell);
   }
   else
+#endif
   {
      KDiff3Shell* p = new KDiff3Shell();
      p->show();
+     p->setWindowState( p->windowState() | Qt::WindowActive ); // Patch for ubuntu: window not active on startup
   }
 //app.installEventFilter( new CFilter );
   int retVal = app.exec();
+  if (QApplication::clipboard()->text().size() == 0)
+     QApplication::clipboard()->clear(); // Patch for Ubuntu: Fix issue with Qt clipboard
   return retVal;
 }
 
