@@ -19,7 +19,7 @@
  ***************************************************************************/
 
 #include "smalldialogs.h"
-#include "optiondialog.h"
+#include "options.h"
 #include "diff.h"
 
 #include <QComboBox>
@@ -41,7 +41,7 @@
 
 OpenDialog::OpenDialog(
    QWidget* pParent, const QString& n1, const QString& n2, const QString& n3,
-   bool bMerge, const QString& outputName, const char* slotConfigure, OptionDialog* pOptions )
+   bool bMerge, const QString& outputName, const char* slotConfigure, Options* pOptions )
 : QDialog( pParent )
 {
    setObjectName("OpenDialog");
@@ -400,6 +400,12 @@ FindDialog::FindDialog(QWidget* pParent)
    hide();
 }
 
+void FindDialog::setVisible( bool bVisible )
+{
+   QDialog::setVisible( bVisible );
+   m_pSearchString->selectAll();
+   m_pSearchString->setFocus();
+}
 
 RegExpTester::RegExpTester( QWidget* pParent, const QString& autoMergeRegExpToolTip,
    const QString& historyStartRegExpToolTip, const QString& historyEntryStartRegExpToolTip, const QString& historySortKeyOrderToolTip )
